\
\ MagTag Weather Station
\ Written in ESP32forth
\ Concept, design and implementation by: Craig A. Lindley
\ Last Update: 03/26/2023
\

\ ***************************************************************************
\ ***                        Constants and Variables                      ***
\ ***************************************************************************

\ Microseconds in a minute
60000000 constant MIN_TO_USECS

\ Sleep time in minutes
30 constant SLEEP_DURATION_MINS

SLEEP_DURATION_MINS MIN_TO_USECS * constant SLEEP_DURATION

\ ***************************************************************************
\ ***                            Support Words                            ***
\ ***************************************************************************

\ Convert OWM icon string to icon index
: iconToIndex { addr n }

  \ Clear the buffer
  clearFormatBuffer

  \ Copy first 2 chars into buffer
  addr    c@ addChar
  addr 1+ c@ addChar

  getFormatBufferString s>number? drop

  case
     1 of 0 endof
     2 of 1 endof
     3 of 2 endof
     4 of 3 endof
     9 of 4 endof
    10 of 5 endof
    11 of 6 endof
    13 of 7 endof
    50 of 8 endof
  endcase
;

\ Format a time in seconds since the epoch into the format buffer
\ Time is converted to local time before display
: formatLocalTime { secs }

  0 0 { t min0 }

  secs toLocal to t
  t hourFormat12_t addNumber
  [char] : addChar
  t minute_t to min0
  min0 10 <
  if
    s" 0" addString
  then
  min0 addNumber

  t isAM_t 
  if
    0 getAMPM addString
  else
    1 getAMPM addString
  then
;

\ Display the date string at specified location on display
: displayDate ( x y )

  2 setTextSize

  \ Clear format buffer
  clearFormatBuffer

  now toLocal >r
  r@ weekDay_t getDayName addString 
  addSpace
  r@ month_t getMonName addString
  addSpace
  r@ day_t addNumber
  addSpace 
  r> year_t addNumber  

  \ Display the date
  getFormatBufferString pString
;

\ Display the city at specified location on display
: displayCityandTime ( x y -- )

  \ Clear format buffer
  clearFormatBuffer

  \ Place city name into format buffer
  CITY_BUF getString addString

  \ Add separator
  s"   --  " addString

  now formatLocalTime
  
  1 setTextSize
  getFormatBufferString pString
;

\ Display the current conditions icon at specified location on display
: displayIcon ( x y -- )

  ICON_BUF getString iconToIndex -rot draw20pxIcon
;

\ Display current temp
: displayTemp ( x y -- )

  \ Clear format buffer
  clearFormatBuffer

  TEMP_BUF getString addString
  s" F" addString

  2 setTextSize
  getFormatBufferString pString
;

\ Display wind speed
: displayWindSpeed ( x y -- )

  \ Clear format buffer
  clearFormatBuffer

  SPEED_BUF getString  addString
  s" MPH" addString

  2 setTextSize
  getFormatBufferString pString
;

\ Display humidity
: displayHumidity ( x y -- )

  \ Clear format buffer
  clearFormatBuffer

  HUM_BUF getString addString
  s" %" addString

  2 setTextSize
  getFormatBufferString pString
;

\ Display sunrise time
: displaySunRiseTime ( x y )

  \ Clear format buffer
  clearFormatBuffer

  \ Convert time string to a number
  SUNRISE_BUF getString s>number? drop
  
  formatLocalTime

  2 setTextSize
  getFormatBufferString pString
;

\ Display sunset time
: displaySunSetTime ( x y )

  \ Clear format buffer
  clearFormatBuffer

  \ Convert time string to a number
  SUNSET_BUF getString s>number? drop
  
  formatLocalTime

  2 setTextSize
  getFormatBufferString pString
;

\ Display future day labels
: displayFutureDayLabels ( -- )

  2 setTextSize

  now toLocal weekDay_t { wd } \ 1 .. 7
  1 +to wd
  wd 7 > if 1 to wd then
  193 30 wd getDayName pString
  1 +to wd
  wd 7 > if 1 to wd then
  193 50 wd getDayName pString
  1 +to wd
  wd 7 > if 1 to wd then
  193 70 wd getDayName pString
  1 +to wd
  wd 7 > if 1 to wd then
  193 90 wd getDayName pString
;

\ Display future day icons
: displayFutureDayIcons ( -- )

  ICON1_BUF getString iconToIndex 232 28 draw20pxIcon
  ICON2_BUF getString iconToIndex 232 48 draw20pxIcon
  ICON3_BUF getString iconToIndex 232 68 draw20pxIcon
  ICON4_BUF getString iconToIndex 232 88 draw20pxIcon
;

\ Display future day noon temps
: displayFutureDayTemps ( -- )

  0 0 { addr t }

  \ Clear format buffer
  clearFormatBuffer

  TEMP1_BUF getString drop to addr
  
  \ Copy first 2 chars into buffer
  addr    c@ addChar
  addr 1+ c@ addChar

  getFormatBufferString s>number? drop to t

  clearFormatBuffer

  t 99 >
  if
    s" HOT" addString
  else 
    t 0 <
    if
      s" BRR" addString
    else
      t str addString
      s" F" addString
    then
  then
  256 30 getFormatBufferString pString

  \ Clear format buffer
  clearFormatBuffer

  TEMP2_BUF getString drop to addr
  
  \ Copy first 2 chars into buffer
  addr    c@ addChar
  addr 1+ c@ addChar

  getFormatBufferString s>number? drop to t
  clearFormatBuffer

  t 99 >
  if
    s" HOT" addString
  else 
    t 0 <
    if
      s" BRR" addString
    else
      t str addString
      s" F" addString
    then
  then
  256 50 getFormatBufferString pString

  \ Clear format buffer
  clearFormatBuffer

  TEMP3_BUF getString drop to addr
  
  \ Copy first 2 chars into buffer
  addr    c@ addChar
  addr 1+ c@ addChar

  getFormatBufferString s>number? drop to t
  clearFormatBuffer

  t 99 >
  if
    s" HOT" addString
  else 
    t 0 <
    if
      s" BRR" addString
    else
      t str addString
      s" F" addString
    then
  then
  256 70 getFormatBufferString pString

  \ Clear format buffer
  clearFormatBuffer

  TEMP4_BUF getString drop to addr
  
  \ Copy first 2 chars into buffer
  addr    c@ addChar
  addr 1+ c@ addChar

  getFormatBufferString s>number? drop to t
  clearFormatBuffer

  t 99 >
  if
    s" HOT" addString
  else 
    t 0 <
    if
      s" BRR" addString
    else
      t str addString
      s" F" addString
    then
  then
  256 90 getFormatBufferString pString
;

\ ***************************************************************************
\ ***                            Program Entry                            ***
\ ***************************************************************************

WiFi

\ Test code entry point
: main ( -- )

  \ Prepare to go to sleep
  SLEEP_DURATION Sleep.timerWakeupUS

  \ Initialize MagTag display
  3 initEPD

  100 delay

  \ Login to WiFi network
  SSID PSWD login

  100 delay

  \ Check for Internet availability by pinging google DNS
  z" 8.8.8.8" WiFi.hostByName 1 =
  if
    \ Internet is available

    \ Set US Mountain timezone
    usMT setTZ

    \ Prime the time code
    now toLocal drop

    \ Draw left most rounded rect
    1 1 186 126 10 EPD_BLACK roundedRect
    2 2 184 124 10 EPD_BLACK roundedRect

    \ Draw right most rounded rect
    189 1 107 126 10 EPD_BLACK roundedRect
    190 2 105 124 10 EPD_BLACK roundedRect

    2 setTextSize

    \ Display static labels
    65 5 s" Today" pString
    208 5 s" Future" pString
    65 48 s" T:" pString
    65 63 s" W:" pString
    65 78 s" H:" pString
    5  93 s" SunRis:" pString
    5 108 s" SunSet:" pString  

    \ Gather the current conditions weather data
    gatherOWMCCData

    500 delay

    \ Gather forecasted weather data
    gatherOWMFData    

    \ Display the date
    5 22 displayDate

    \ Display the city and the current time
    5 38 displayCityandTime

    \ Display current conditions icon
    25 60 displayIcon

    \ Display temp
    88 48 displayTemp

    \ Display wind speed
    88 63 displayWindSpeed

    \ Display humidity
    88 78 displayHumidity

    \ Display sunrise and sunset time
    88  93 displaySunRiseTime
    88 108 displaySunSetTime

    \ Display future day labels
    displayFutureDayLabels

    \ Display future day icons
    displayFutureDayIcons

    \ Display future day noon temps
    displayFutureDayTemps

    \ Update display 
    display
  then
  
  \ Go to sleep
  Sleep.deepSleep
;

forth
